"""
class QTColor defines the recognize SVG color keywords

colors https://en.wikipedia.org/wiki/Web_colors

"""

from PySide2.QtGui import QColor

class SvgColor:
    """
    Standard SVG color labels
    """

    aliceblue = QColor(240, 248, 255)
    antiquewhite = QColor(250, 235, 215)
    aqua = QColor(0, 255, 255)
    aquamarine = QColor(127, 255, 212)
    azure = QColor(240, 255, 255)
    beige = QColor(245, 245, 220)
    bisque = QColor(255, 228, 196)
    black = QColor(0, 0, 0)
    blanchedalmond = QColor(255, 235, 205)
    blue = QColor(0, 0, 255)
    blueviolet = QColor(138, 43, 226)
    brown = QColor(165, 42, 42)
    burlywood = QColor(222, 184, 135)
    cadetblue = QColor(95, 158, 160)
    chartreuse = QColor(127, 255, 0)
    chocolate = QColor(210, 105, 30)
    coral = QColor(255, 127, 80)
    cornflowerblue = QColor(100, 149, 237)
    cornsilk = QColor(255, 248, 220)
    crimson = QColor(220, 20, 60)
    cyan = QColor(0, 255, 255)
    darkblue = QColor(0, 0, 139)
    darkcyan = QColor(0, 139, 139)
    darkgoldenrod = QColor(184, 134, 11)
    darkgray = QColor(169, 169, 169)
    darkgreen = QColor(0, 100, 0)
    darkgrey = QColor(169, 169, 169)
    darkkhaki = QColor(189, 183, 107)
    darkmagenta = QColor(139, 0, 139)
    darkolivegreen = QColor(85, 107, 47)
    darkorange = QColor(255, 140, 0)
    darkorchid = QColor(153, 50, 204)
    darkred = QColor(139, 0, 0)
    darksalmon = QColor(233, 150, 122)
    darkseagreen = QColor(143, 188, 143)
    darkslateblue = QColor(72, 61, 139)
    darkslategray = QColor(47, 79, 79)
    darkslategrey = QColor(47, 79, 79)
    darkturquoise = QColor(0, 206, 209)
    darkviolet = QColor(148, 0, 211)
    deeppink = QColor(255, 20, 147)
    deepskyblue = QColor(0, 191, 255)
    dimgray = QColor(105, 105, 105)
    dimgrey = QColor(105, 105, 105)
    dodgerblue = QColor(30, 144, 255)
    firebrick = QColor(178, 34, 34)
    floralwhite = QColor(255, 250, 240)
    forestgreen = QColor(34, 139, 34)
    fuchsia = QColor(255, 0, 255)
    gainsboro = QColor(220, 220, 220)
    ghostwhite = QColor(248, 248, 255)
    gold = QColor(255, 215, 0)
    goldenrod = QColor(218, 165, 32)
    gray = QColor(128, 128, 128)
    grey = QColor(128, 128, 128)
    green = QColor(0, 128, 0)
    greenyellow = QColor(173, 255, 47)
    honeydew = QColor(240, 255, 240)
    hotpink = QColor(255, 105, 180)
    indianred = QColor(205, 92, 92)
    indigo = QColor(75, 0, 130)
    ivory = QColor(255, 255, 240)
    khaki = QColor(240, 230, 140)
    lavender = QColor(230, 230, 250)
    lavenderblush = QColor(255, 240, 245)
    lawngreen = QColor(124, 252, 0)
    lemonchiffon = QColor(255, 250, 205)
    lightblue = QColor(173, 216, 230)
    lightcoral = QColor(240, 128, 128)
    lightcyan = QColor(224, 255, 255)
    lightgoldenrodyellow = QColor(250, 250, 210)
    lightgray = QColor(211, 211, 211)
    lightgreen = QColor(144, 238, 144)
    lightgrey = QColor(211, 211, 211)
    lightpink = QColor(255, 182, 193)
    lightsalmon = QColor(255, 160, 122)
    lightseagreen = QColor(32, 178, 170)
    lightskyblue = QColor(135, 206, 250)
    lightslategray = QColor(119, 136, 153)
    lightslategrey = QColor(119, 136, 153)
    lightsteelblue = QColor(176, 196, 222)
    lightyellow = QColor(255, 255, 224)
    lime = QColor(0, 255, 0)
    limegreen = QColor(50, 205, 50)
    linen = QColor(250, 240, 230)
    magenta = QColor(255, 0, 255)
    maroon = QColor(128, 0, 0)
    mediumaquamarine = QColor(102, 205, 170)
    mediumblue = QColor(0, 0, 205)
    mediumorchid = QColor(186, 85, 211)
    mediumpurple = QColor(147, 112, 219)
    mediumseagreen = QColor(60, 179, 113)
    mediumslateblue = QColor(123, 104, 238)
    mediumspringgreen = QColor(0, 250, 154)
    mediumturquoise = QColor(72, 209, 204)
    mediumvioletred = QColor(199, 21, 133)
    midnightblue = QColor(25, 25, 112)
    mintcream = QColor(245, 255, 250)
    mistyrose = QColor(255, 228, 225)
    moccasin = QColor(255, 228, 181)
    navajowhite = QColor(255, 222, 173)
    navy = QColor(0, 0, 128)
    oldlace = QColor(253, 245, 230)
    olive = QColor(128, 128, 0)
    olivedrab = QColor(107, 142, 35)
    orange = QColor(255, 165, 0)
    orangered = QColor(255, 69, 0)
    orchid = QColor(218, 112, 214)
    palegoldenrod = QColor(238, 232, 170)
    palegreen = QColor(152, 251, 152)
    paleturquoise = QColor(175, 238, 238)
    palevioletred = QColor(219, 112, 147)
    papayawhip = QColor(255, 239, 213)
    peachpuff = QColor(255, 218, 185)
    peru = QColor(205, 133, 63)
    pink = QColor(255, 192, 203)
    plum = QColor(221, 160, 221)
    powderblue = QColor(176, 224, 230)
    purple = QColor(128, 0, 128)
    red = QColor(255, 0, 0)
    rosybrown = QColor(188, 143, 143)
    royalblue = QColor(65, 105, 225)
    saddlebrown = QColor(139, 69, 19)
    salmon = QColor(250, 128, 114)
    sandybrown = QColor(244, 164, 96)
    seagreen = QColor(46, 139, 87)
    seashell = QColor(255, 245, 238)
    sienna = QColor(160, 82, 45)
    silver = QColor(192, 192, 192)
    skyblue = QColor(135, 206, 235)
    slateblue = QColor(106, 90, 205)
    slategray = QColor(112, 128, 144)
    snow = QColor(255, 250, 250)
    springgreen = QColor(0, 255, 127)
    steelblue = QColor(70, 130, 180)
    tan = QColor(210, 180, 140)
    teal = QColor(0, 128, 128)
    thistle = QColor(216, 191, 216)
    tomato = QColor(255, 99, 71)
    turquoise = QColor(64, 224, 208)
    violet = QColor(238, 130, 238)
    wheat = QColor(245, 222, 179)
    white = QColor(255, 255, 255)
    whitesmoke = QColor(245, 245, 245)
    yellow = QColor(255, 255, 0)
    yellowgreen = QColor(154, 205, 50)
