"""
miscellaneous functions/classes
"""

from .classes import XLate

from .cipher import encrypt, decrypt
from .decorators import staticVars, callCounter
from .regexutils import multipleReplace
from .strFormatTimeDelta import strFormatTimeDelta
