"""VS module names"""

from .mkvattachments import MKVAttachment, MKVAttachments
from .mkvclassutil import SourceFile, SourceFiles
from .mkvcommand import MKVCommand
#from .mkvcommandNew import MKVCommandNew
from .mkvcommandparser import MKVCommandParser, MKVParseKey
from .verifystructure import VerifyStructure
from .verifycommand import VerifyMKVCommand
