"""
classes related to files
========================

ConfigurationSettings - read/save dictionary of terms
    from file, keys must be strings

"""

from .configurationsettings import ConfigurationSettings
from .DisplayPath import DisplayPath
